"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@babel/core");
const helper_annotate_as_pure_1 = __importDefault(require("@babel/helper-annotate-as-pure"));
const helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
exports.default = helper_plugin_utils_1.declare((api, options) => {
    api.assertVersion(7);
    const { allowMutablePropsOnTags } = options;
    const HOISTED = new WeakSet();
    const immutabilityVisitor = {
        enter(path, state) {
            const stop = () => {
                state.isImmutable = false;
                path.stop();
            };
            if (path.isJSXClosingElement()) {
                path.skip();
                return;
            }
            if (path.isJSXIdentifier({ name: 'ref' }) &&
                path.parentPath.isJSXAttribute({ name: path.node })) {
                return stop();
            }
            if (path.isJSXIdentifier() || path.isIdentifier() || path.isJSXMemberExpression()) {
                return;
            }
            if (!path.isImmutable()) {
                if (path.isPure()) {
                    const expressionResult = path.evaluate();
                    if (expressionResult.confident) {
                        const { value } = expressionResult;
                        const isMutable = (!state.mutablePropsAllowed && (value && typeof value === 'object')) ||
                            typeof value === 'function';
                        if (!isMutable) {
                            path.skip();
                            return;
                        }
                    }
                    else if (core_1.types.isIdentifier(expressionResult.deopt)) {
                        return;
                    }
                }
                stop();
            }
        },
    };
    let IMPORTS = [];
    let hasExited = false;
    return {
        name: 'transform-react-constant-elements',
        visitor: {
            ImportDeclaration(path) {
                if (!hasExited) {
                    IMPORTS.push(path.node);
                    path.remove();
                }
            },
            Program: {
                enter() {
                    hasExited = false;
                    IMPORTS = [];
                },
                exit(path) {
                    hasExited = true;
                    path.node.body = [...IMPORTS, ...path.node.body];
                },
            },
            JSXElement(path) {
                if (HOISTED.has(path.node))
                    return;
                HOISTED.add(path.node);
                const state = { isImmutable: true };
                if (allowMutablePropsOnTags) {
                    let namePath = path.get('openingElement.name');
                    while (namePath.isJSXMemberExpression()) {
                        namePath = namePath.get('property');
                    }
                    const elementName = namePath.node.name;
                    state.mutablePropsAllowed = Array.isArray(allowMutablePropsOnTags)
                        ? allowMutablePropsOnTags.indexOf(elementName) > -1
                        : true;
                }
                path.traverse(immutabilityVisitor, state);
                if (state.isImmutable) {
                    const hoisted = path.hoist();
                    if (hoisted) {
                        helper_annotate_as_pure_1.default(hoisted);
                    }
                }
            },
        },
    };
});
//# sourceMappingURL=index.js.map